#ifndef MESH_H
#define MESH_H

#include <cstddef>
#include <string>
#include <vector>
#include <cstdint>
#include <QString>
#include "vec.h"
#include "matrix.h"

class QGLShaderProgram;
class KdTree;

class Mesh
{
	friend class Parser3DS;
public:
	Mesh();
	~Mesh();

	void clear();

	void draw();

	void load(const QString &filename);

	std::pair<Vec3, Vec3> getBoundingBox() const;
	void setBoundingBox(const std::pair<Vec3, Vec3> &bbox);

	void swap(Mesh *mesh);

	void transform(const Mat44 &m);

    void mergeChilds();

    void mergeIdenticalVertices();

    void triangulate();

    void slice(const double z, const double h, std::vector<Vec2d> &lines, const int line_id) const;
	static Vec2d optimizeSlice(const Vec2d &start, std::vector<Vec2d> &lines);

    void buildKdTree();

	void buildSupportHull();

	void computeFaceNormals();

	void backup();

	void invertNormals();

protected:
	void recompute_normals();
	void loadOFF(const QString &filename);
	void loadOBJ(const QString &filename);
	void loadSTL(const QString &filename);
	void load3DS(const QString &filename);
	void loadPLY(const QString &filename);

protected:
	std::string name;
	std::vector<Vec3> vertex;
	std::vector<Vec3> vertex_orig;
	std::vector<Vec3> normal;
	std::vector<u_int32_t> index;
	u_int32_t primitive_type;
	std::vector<Mesh*> childs;

	std::vector<Vec3> base_vertex;
	std::vector<u_int32_t> base_index;

	std::vector<Vec3> hull_vertex;
	std::vector<u_int32_t> hull_index;
	std::vector<Vec3> face_normal;

    KdTree *kdtree;
	KdTree *hull_kdtree;
	KdTree *base_kdtree;

	mutable std::pair<Vec3, Vec3> bbox;
	mutable bool b_bbox_dirty;
	Mat44 local;
};

#endif // MESH_H
