#include "mesh.h"
#include "filereader.h"
#include "utils.h"
#include <unordered_map>
#include <QGLContext>
#include <QFileInfo>
#include <QDebug>
#include <QHash>

using namespace std;

namespace std
{
	template<typename T, typename U>
	struct hash<pair<T,U> >
	{
	public:
		inline size_t operator()(const pair<T,U> &v) const
		{
			return (h0(v.first) * 31U) ^ h1(v.second);
		}
	private:
		std::hash<T> h0;
		std::hash<U> h1;
	};
}

void Mesh::loadOBJ(const QString &filename)
{
	FileReader file(filename.toStdString());
	if (!file.isOpen())
		return;

	primitive_type = GL_TRIANGLES;

	Mesh *cur = this;

	vector<Vec3> t_vtx;
	vector<Vec2> t_tcd;
	vector<char*> elts;
	vector<char*> components;
	unordered_map<pair<int, int>, size_t> elts_table;

	string word;
	while(!file.atEnd())
	{
		file.skipWhitespaces();
		int c = file.peek();
		switch(c)
		{
		case '#':
			file.skipLine();
			continue;
		case 'u':
		case 'g':
		case 'o':
			file >> word;
			if (word == "o" || word == "g" || word == "usemtl")
			{
				if (word == "usemtl")
				{
					file.skipWhitespaces();
					word = file.getLine();
					word = cur->name + '_' + word;
				}
				else
					word = file.getLine();
				childs.push_back(new Mesh);
				cur = childs.back();
				cur->name = word;
				cur->primitive_type = GL_TRIANGLES;
				elts_table.clear();
			}
			break;
		case 'v':
			file >> word;
			if (word == "v")
			{
				Vec3 p;
				file >> p.x() >> p.y() >> p.z();
				t_vtx.push_back(p);
			}
			else if (word == "vn")
			{
				Vec3 n;
				file >> n.x() >> n.y() >> n.z();
			}
			else if (word == "vt")
			{
				Vec2 t;
                double tmp;
				file >> t.x() >> t.y() >> tmp;
				t_tcd.push_back(t);
			}
			break;
		case 'm':
			file >> word;
			if (word == "mtllib")
			{
				word = file.getLine();
			}
			break;
		case 'f':
			file >> word;
			if (word == "f")
			{
				file.skipWhitespaces();
				char buf[1024];
				file.getSplitLine(buf, 1024);
				elts.clear();
				split(buf, ' ', elts, false);
				size_t a = 0, b = 0;
				for(size_t i = 0 ; i < elts.size() ; ++i)
				{
					components.clear();
					split(elts[i], '/', components);
					int v_id(0);
					int t_id(-1);
					v_id = parse_int(components[0]) - 1;
					if (components.size() > 1)	t_id = parse_int(components[1]) - 1;
					const pair<int,int> p(v_id, t_id);
					size_t idx;
					unordered_map<pair<int, int>, size_t>::const_iterator it = elts_table.find(p);
					if (it == elts_table.end())
					{
						idx = cur->vertex.size();
						elts_table.insert(make_pair(p, idx));
						cur->vertex.push_back(t_vtx[v_id]);
					}
					else
						idx = it->second;
					if (i == 0)
						a = idx;
					else if (i > 1)
					{
						cur->index.push_back(a);
						cur->index.push_back(b);
						cur->index.push_back(idx);
					}
					b = idx;
				}
			}
			break;
		case 's':
			file >> word;
			if (word == "s")
			{
				file.skipLine();
			}
			break;
		default:
			file.skipLine();
		}
	}

	recompute_normals();
}
