#include "mesh.h"
#include <QFile>
#include <QTextStream>
#include <QDebug>
#include <QGLWidget>

void Mesh::loadSTL(const QString &filename)
{
	QFile file(filename);

	file.open(QIODevice::ReadOnly | QIODevice::Text);
	if (!file.isOpen())
		return;

	primitive_type = GL_TRIANGLES;

	QTextStream stream(&file);
	const QString &head = stream.readLine();
    if (head.left(6) == "solid " && head.size() < 80)	// ASCII format
	{
		name = head.right(head.size() - 6).toStdString();
		QString word;
		stream >> word;
		for(; word == "facet" ; stream >> word)
		{
			stream >> word;	// normal
			Vec3 n;
			stream >> n[0] >> n[1] >> n[2];
			n.normalize();

			stream >> word >> word;	// outer loop
			stream >> word;
			size_t start_idx = vertex.size();
			for(; word != "endloop" ; stream >> word)
			{
				Vec3 v;
				stream >> v[0] >> v[1] >> v[2];
				vertex.push_back(v);
			}
			for(size_t i = start_idx + 2 ; i < vertex.size() ; ++i)
			{
				index.push_back(start_idx);
				index.push_back(i - 1);
				index.push_back(i);
			}
			stream >> word;	// endfacet
		}
	}
	else			// BINARY format
	{
		file.setTextModeEnabled(false);
		file.seek(80);
		u_int32_t nb_triangles;
		file.read((char*)&nb_triangles, sizeof(nb_triangles));
		vertex.reserve(nb_triangles * 3U);
		normal.reserve(nb_triangles * 3U);
		index.reserve(nb_triangles * 3U);
		for(size_t i = 0 ; i < nb_triangles ; ++i)
		{
            Vec3 n, a, b, c;
#define READ_VECTOR(v)\
			do {\
				float f;\
				file.read((char*)&f,sizeof(float));	v[0] = f;\
				file.read((char*)&f,sizeof(float));	v[1] = f;\
				file.read((char*)&f,sizeof(float));	v[2] = f;\
			} while(false)

			READ_VECTOR(n);
			READ_VECTOR(a);
			READ_VECTOR(b);
			READ_VECTOR(c);

			index.push_back(vertex.size());
			index.push_back(vertex.size() + 1);
			index.push_back(vertex.size() + 2);
			vertex.push_back(a);
			vertex.push_back(b);
			vertex.push_back(c);

			u_int16_t attribute_byte_count;
			file.read((char*)&attribute_byte_count, sizeof(attribute_byte_count));
		}
	}
	recompute_normals();
}
